/*  Feeds - NEWS
        
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    The primary data model for the News app. Feeds includes the primary
        data structure for the newsfeeds, which are structured as a list of lists:

    Feeds.list entry is:
      list[x].title           String    Title entered by user
      list[x].url             String    Feed source URL in unescaped form
      list[x].type            String    Feed type: either rdf, rss or atom
      list[x].numUnRead       Integer   How many stories are still unread
      list[x].newStoryCount   Integer   For each update, how many new stories
      list[x].stories         Array     Each entry is a complete story
          
    list.stories entry is:    
       stories[y].title       String    Story title or headline
       stories[y].text        String    Story text
       stories[y].summary     String    Story text, stripped of markup
       stories[y].unreadStyle String    Null when Read
       stories[y].url         String    Story url                

    Methods:
    initialize(test) - create default and test feed lists
    getDefaultList() - returns the default feed list as an array
    getTestList() - returns both the default and test feed lists as a single array
    processFeed(transport, index) - function to process incoming feeds that are
        XML encoded in an Ajax object and stores them in the Feeds.list. Supports
        RSS, RDF and Atom feed formats.
    updateFeedList(index) - updates entire feed list starting with this.feedIndex.
*/  

var Feeds = Class.create ({
    
    //  Default Feeds.list
    defaultList: [
        {
            title: "New York Times",
            url:"http://www.nytimes.com/services/xml/rss/nyt/HomePage.xml",
            type:"rss",
            numUnRead:4,
            stories:[
              {
                  title: "Obama Warns of Prospect for Trillion-Dollar Deficits",
                  text: "Barack Obama delivered a stark assessment of the economy, saying that his administration would be forced to impose tighter discipline on government.",
                  unreadStyle: "unReadStyle",
                  url: "http://www.nytimes.com/2009/01/07/world/asia/07india.html?_r=1&partner=rss&emc=rss"
              },
              {
                  title: "Hundreds of Coal Ash Dumps Lack Regulation",
                  text: "Most of the coal byproduct dumps across the United States are unregulated, although they contain chemicals considered as threats to human health.",
                  unreadStyle: "unReadStyle",
                  url: "http://www.nytimes.com/2009/01/06/world/asia/06iqbal.html?partner=rss&emc=rss"
              },
              {
                  title: "Gazprom Dispute Entangles Europe",
                  text: "Russia’s gas price dispute with Ukraine escalated, disrupting deliveries to the European Union in the midst of a bitter cold spell.",
                  unreadStyle: "unReadStyle",
                  url: "http://www.nytimes.com/2009/01/07/world/europe/07gazprom.html?partner=rss&emc=rss"
              },
              {
                  title: "Green Inc.: Cities Target Lending to Speed Energy Projects",
                  text: "A number of municipalities across the country are getting creative and experimenting with incremental, neighborhood- or district-based lending programs that help homeowners pay the up-front capital costs for efficiency or renewable energy projects.",
                  unreadStyle: "unReadStyle",
                  url: "http://greeninc.blogs.nytimes.com/2009/01/06/cities-use-creative-targeted-lending-to-speed-energy-projects/?partner=rss&emc=rss"
              }
            ]
        }

    ],
        
    // initialize - Assign default data to the feedlist 
    initialize: function(test)  {
        this.list = this.getDefaultList();
    },
    
    // getDefaultList - returns the default feed list as an array
    getDefaultList: function() {
        var returnList = [];
        for (var i=0; i<this.defaultList.length; i++)   {
            returnList[i] = this.defaultList[i];
        }
        
        return returnList;
    }

});
    