/*  StageAssistant - NEWS
    
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Responsible for app startup.
    Major components:
    - setup; app startup including preferences, initial load of default 
        feed data and setting alarms for periodic feed updates
    
    Data structures:
    - globals; set of persistant data used throughout app
    - Feeds Model; handles all feedlist updates and default data
        
    App architecture:
    - StageAssistant; handles startup and feed list management
    - FeedListAssistant; handles feedList navigation
    - StoryListAssistant; handles single feed navigation
    - StoryViewAssistant; handles single story navigation

*/  

//  ---------------------------------------------------------------
//    GLOBALS
//  ---------------------------------------------------------------

//  News namespace
News = {};

// Constants
News.unreadStory = "unReadStyle";                                    
News.versionString = "1.0";
News.errorNone =    "0";                    // No error, success
News.invalidFeedError = "1";                // Not RSS2, RDF (RSS1), or ATOM

// Global Data Structures

// Persistent Globals - will be saved across app launches
News.feedUpdateInterval = 900000;           // Feed update interval 

//  Session Globals - not saved across app launches
News.feedListChanged = false;               // Triggers update to Depot db
News.feedListUpdateInProgress = false;      // Feed update is in progress
News.dbUpdate = "";                         // Default is no update
News.wakeupTaskId = undefined;              // Id for wakeup tasks


function StageAssistant () {

}

//  -------------------------------------------------------
//  setup - all startup actions:
//    - Setup globals
//    - Initiate alarm for first feed update                    

StageAssistant.prototype.setup = function() {
    
    // initialize the feeds model and update the feeds
    this.feeds = new Feeds();
    
    // Set up first timeout alarm
    this.setWakeup();
    
    // Push the first scene
    this.controller.pushScene("feedList", this.feeds);
    
};

// ------------------------------------------------------------------------
// handleWakeup - called when wakeup timer fires; sets a new timer and calls
//   for a feed update cycle
StageAssistant.prototype.handleWakeup = function() {    

    // Set next wakeup alarm
    this.setWakeup();

    // Update the feed list
    Mojo.Log.info("Update FeedList");
    this.feeds.updateFeedList();
    
};

// ------------------------------------------------------------------------
// setWakeup - called to setup the wakeup alarm for background feed updates
//   if preferences are not set for a manual update (value of 0)
StageAssistant.prototype.setWakeup = function() {    

    if (News.feedUpdateInterval !== 0)    {
        var interval = News.feedUpdateInterval;
        News.wakeupTaskId = this.controller.window.setTimeout(this.handleWakeup.bind(this), interval);
        Mojo.Log.info("Set Interval Timer: ", interval);
    }
    
};

// ------------------------------------------------------------------------
// cleanup - clear the wakeup alarm for background feed updates if set
StageAssistant.prototype.cleanup = function() {
    if (News.wakeupTaskId !== 0)    {
       News.wakeupTaskId = this.controller.window.clearTimeout(News.wakeupTaskId);
       Mojo.Log.info("clear Interval Timer");
    }
};
