/*  StoryViewAssistant - NEWS
        
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Passed a story element, displays that element in a full scene view and offers 
    button options for next story or previous story
    Major components:
    - StoryView; display story in main scene
    - Next/Previous; button options to go to next or previous story
    
    Arguments:
    - storyFeed; Selected feed from which the stories are being viewed
    - storyIndex; Index of selected story to be put into the view                
*/

function StoryViewAssistant(storyFeed, storyIndex) {
    this.storyFeed = storyFeed;
    this.storyIndex = storyIndex;
}

// setup
StoryViewAssistant.prototype.setup = function() {    
    this.nextModel = {disabled: false};
    this.previousModel= {disabled: false};
    
    if (this.storyIndex === 0)    {
        this.previousModel.disabled = true;
    }
    
    if (this.storyIndex == this.storyFeed.stories.length-1)    {
        this.nextModel.disabled = true;
    }
    
    this.controller.setupWidget("previousStory", {label: "Previous"}, this.previousModel);
    this.controller.setupWidget("nextStory", {label: "Next"}, this.nextModel);

    this.nextStoryHandler = this.nextStory.bindAsEventListener(this);
    this.previousStoryHandler = this.previousStory.bindAsEventListener(this);
    this.controller.listen('nextStory', Mojo.Event.tap, this.nextStoryHandler);
    this.controller.listen('previousStory', Mojo.Event.tap, this.previousStoryHandler);
    
    // Update story title in header and summary
    var storyViewTitleElement = this.controller.get("storyViewTitle");
    var storyViewSummaryElement = this.controller.get("storyViewSummary");
    storyViewTitleElement.innerHTML = this.storyFeed.stories[this.storyIndex].title;
    storyViewSummaryElement.innerHTML = this.storyFeed.stories[this.storyIndex].text;
};

// activate - display selected story
StoryViewAssistant.prototype.activate = function(event) {
    Mojo.Log.info("Story View Activated");
    
    // Update unreadStyle string and unReadCount in case it's changed
    if (this.storyFeed.stories[this.storyIndex].unreadStyle == News.unreadStory)    {
        this.storyFeed.numUnRead--;
        this.storyFeed.stories[this.storyIndex].unreadStyle = "";                                
        News.feedListChanged = true;
    } 

};

// cleanup - remove listeners
StoryViewAssistant.prototype.cleanup = function(event) {
    this.controller.stopListening('nextStory', Mojo.Event.tap, this.nextStoryHandler);
    this.controller.stopListening('previousStory', Mojo.Event.tap, this.previousStoryHandler);
};

StoryViewAssistant.prototype.previousStory = function(event) {
    Mojo.Controller.stageController.swapScene(
        {
            transition: Mojo.Transition.crossFade,
            name: "storyView"
        },
        this.storyFeed, this.storyIndex-1);
};

StoryViewAssistant.prototype.nextStory = function(event) {
    Mojo.Controller.stageController.swapScene(
        {
            transition: Mojo.Transition.crossFade,
            name: "storyView"
        },
        this.storyFeed, this.storyIndex+1);
};



