/*  Preferences - NEWS
        
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Preferences - Handles preferences scene, where the user can:
        - enable or disable the featured feed element on the FeedList Scene
        - select the featured feed when enabled
        - select the featured feed rotation interval
        - select the interval for feed updates
        - enable or disable background feed notifications
        - enable or disable device wakeup
    
    App Menu is disabled in this scene.
        
*/    

function PreferencesAssistant() {

}

PreferencesAssistant.prototype.setup = function() {
        
    // Setup Integer Picker to pick feature feed rotation interval
    this.controller.setupWidget("featureFeedDelay",
        {
            label:    "Rotation (in seconds)",
            modelProperty:    "value", 
            min: 1,
            max: 20    
        },        
        this.featureDelayModel = {
            value : News.featureStoryInterval/1000
        });

    this.changeFeatureDelayHandler = this.changeFeatureDelay.bindAsEventListener(this);
    this.controller.listen("featureFeedDelay", Mojo.Event.propertyChange,
        this.changeFeatureDelayHandler);
    
    // Setup list selector for UPDATE INTERVAL
    this.controller.setupWidget("feedCheckIntervalList",
        {
            label: "Interval",
            choices: [
                {label: "Manual Updates",   value: 0},
                {label: "5 Minutes",        value: 300000},    
                {label: "15 Minutes",       value: 900000},    
                {label: "1 Hour",           value: 3600000},    
                {label: "4 Hours",          value: 14400000},
                {label: "1 Day",            value: 86400000}
            ]    
        },
        this.feedIntervalModel = {
            value : News.feedUpdateInterval
        });
    
    this.changeFeedIntervalHandler = this.changeFeedInterval.bindAsEventListener(this);
    this.controller.listen("feedCheckIntervalList", Mojo.Event.propertyChange,
        this.changeFeedIntervalHandler);
};

// Deactivate - save News preferences and globals
PreferencesAssistant.prototype.deactivate = function() {
    News.Cookie.storeCookie();                                                                                    
};

// Cleanup - remove listeners
PreferencesAssistant.prototype.cleanup = function() {
    this.controller.stopListening("featureFeedDelay", 
        Mojo.Event.propertyChange, this.changeFeatureDelayHandler);
    this.controller.stopListening("feedCheckIntervalList", 
        Mojo.Event.propertyChange, this.changeFeedIntervalHandler);
};

//    changeFeatureDelay - Handle changes to the feature feed interval
PreferencesAssistant.prototype.changeFeatureDelay = function(event) {
    Mojo.Log.info("Preferences Feature Delay Handler; value = ",
        this.featureDelayModel.value);
    
    //  Interval is in milliseconds
    News.featureStoryInterval = this.featureDelayModel.value*1000;
    
    // If timer is active, restart with new value
    if(News.featureStoryTimer !== null) {
        this.controller.window.clearInterval(News.featureStoryTimer);
        News.featureStoryTimer = null;
    }                                                        
};

//    changeFeedInterval    - Handle changes to the feed update interva;
PreferencesAssistant.prototype.changeFeedInterval = function(event) {
    Mojo.Log.info("Preferences Feed Interval Handler; value = ",
        this.feedIntervalModel.value);
    News.feedUpdateInterval = this.feedIntervalModel.value;
};